"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetLearnModeCallback = exports.SetLearnModeResponse = exports.SetLearnModeRequest = exports.SetLearnModeRequestBase = exports.LearnModeStatus = exports.LearnModeIntent = void 0;
const core_1 = require("@zwave-js/core");
const serial_1 = require("@zwave-js/serial");
const shared_1 = require("@zwave-js/shared");
const LEARN_MODE_EMPTY_NODE_ID = 0xef; // who knows why...
var LearnModeIntent;
(function (LearnModeIntent) {
    LearnModeIntent[LearnModeIntent["LegacyStop"] = 0] = "LegacyStop";
    LearnModeIntent[LearnModeIntent["LegacyInclusionExclusion"] = 1] = "LegacyInclusionExclusion";
    LearnModeIntent[LearnModeIntent["LegacyNetworkWideInclusion"] = 2] = "LegacyNetworkWideInclusion";
    LearnModeIntent[LearnModeIntent["LegacyNetworkWideExclusion"] = 3] = "LegacyNetworkWideExclusion";
    LearnModeIntent[LearnModeIntent["Stop"] = 128] = "Stop";
    // The Z-Wave API Module SHOULD try a direct range inclusion followed by 4 NWI attempts
    LearnModeIntent[LearnModeIntent["Inclusion"] = 129] = "Inclusion";
    LearnModeIntent[LearnModeIntent["DirectExclusion"] = 130] = "DirectExclusion";
    LearnModeIntent[LearnModeIntent["NetworkWideExclusion"] = 131] = "NetworkWideExclusion";
    LearnModeIntent[LearnModeIntent["SmartStart"] = 132] = "SmartStart";
})(LearnModeIntent || (exports.LearnModeIntent = LearnModeIntent = {}));
var LearnModeStatus;
(function (LearnModeStatus) {
    LearnModeStatus[LearnModeStatus["Started"] = 1] = "Started";
    LearnModeStatus[LearnModeStatus["ProtocolDone"] = 5] = "ProtocolDone";
    LearnModeStatus[LearnModeStatus["Completed"] = 6] = "Completed";
    LearnModeStatus[LearnModeStatus["Failed"] = 7] = "Failed";
})(LearnModeStatus || (exports.LearnModeStatus = LearnModeStatus = {}));
let SetLearnModeRequestBase = class SetLearnModeRequestBase extends serial_1.Message {
    constructor(host, options) {
        if ((0, serial_1.gotDeserializationOptions)(options)
            && new.target !== SetLearnModeCallback) {
            return new SetLearnModeCallback(host, options);
        }
        super(host, options);
    }
};
exports.SetLearnModeRequestBase = SetLearnModeRequestBase;
exports.SetLearnModeRequestBase = SetLearnModeRequestBase = __decorate([
    (0, serial_1.messageTypes)(serial_1.MessageType.Request, serial_1.FunctionType.SetLearnMode),
    (0, serial_1.priority)(core_1.MessagePriority.Controller)
], SetLearnModeRequestBase);
let SetLearnModeRequest = class SetLearnModeRequest extends SetLearnModeRequestBase {
    constructor(host, options) {
        super(host, options);
        if ((0, serial_1.gotDeserializationOptions)(options)) {
            throw new core_1.ZWaveError(`${this.constructor.name}: deserialization not implemented`, core_1.ZWaveErrorCodes.Deserialization_NotImplemented);
        }
        else {
            this.intent = options.intent;
        }
    }
    intent;
    serialize() {
        this.payload = Buffer.from([
            this.intent,
            this.callbackId,
        ]);
        return super.serialize();
    }
    toLogEntry() {
        return {
            ...super.toLogEntry(),
            message: {
                "callback id": this.callbackId,
                intent: (0, shared_1.getEnumMemberName)(LearnModeIntent, this.intent),
            },
        };
    }
};
exports.SetLearnModeRequest = SetLearnModeRequest;
exports.SetLearnModeRequest = SetLearnModeRequest = __decorate([
    (0, serial_1.expectedResponse)(serial_1.FunctionType.SetLearnMode)
    // The callback may come much (30+ seconds), so we wait for it outside of the queue
], SetLearnModeRequest);
let SetLearnModeResponse = class SetLearnModeResponse extends serial_1.Message {
    constructor(host, options) {
        super(host, options);
        this.success = this.payload[0] !== 0;
    }
    success;
    isOK() {
        return this.success;
    }
    toLogEntry() {
        return {
            ...super.toLogEntry(),
            message: { success: this.success },
        };
    }
};
exports.SetLearnModeResponse = SetLearnModeResponse;
exports.SetLearnModeResponse = SetLearnModeResponse = __decorate([
    (0, serial_1.messageTypes)(serial_1.MessageType.Response, serial_1.FunctionType.SetLearnMode)
], SetLearnModeResponse);
class SetLearnModeCallback extends SetLearnModeRequestBase {
    constructor(host, options) {
        super(host, options);
        this.callbackId = this.payload[0];
        this.status = this.payload[1];
        this.assignedNodeId = this.payload[2];
        if (this.payload.length > 3) {
            const msgLength = this.payload[3];
            this.statusMessage = this.payload.subarray(4, 4 + msgLength);
        }
    }
    status;
    assignedNodeId;
    statusMessage;
    isOK() {
        return this.status !== LearnModeStatus.Failed;
    }
    toLogEntry() {
        const message = {
            "callback id": this.callbackId,
            status: (0, shared_1.getEnumMemberName)(LearnModeStatus, this.status),
        };
        if (this.status !== LearnModeStatus.Started
            || this.assignedNodeId !== LEARN_MODE_EMPTY_NODE_ID) {
            message["assigned node id"] = this.assignedNodeId;
        }
        if (this.statusMessage?.length) {
            message["status message"] = (0, shared_1.buffer2hex)(this.statusMessage);
        }
        return {
            ...super.toLogEntry(),
            message,
        };
    }
}
exports.SetLearnModeCallback = SetLearnModeCallback;
//# sourceMappingURL=SetLearnModeMessages.js.map